.msgtags {
  margin-left: 10px;
  color: @warning_color;
}

.msgcontext {
  margin-left: 10px;
  color: @insensitive_fg_color;
}

.progress_trans {
  color: @theme_selected_bg_color;
}

.fuzzy,
.progress_fuzzy {
  color: @warning_color;
}

.fuzzy {
  font-style: italic;
}

.untranslated,
.progress_untranslated {
  color: @error_color;
}

.untranslated {
  font-weight: bold;
}

#info_label.error {
  color: @error_color;
}

#info_label.info {
  color: @theme_fg_color;
}

.message-table-row {
  padding-top: 3px;
  padding-bottom: 3px;
}

.message-table-row-header {
  padding-top: 3px;
  padding-bottom: 3px;
}

textview.inline,
textview.inline text {
  background: none;
}

textview.inline {
  padding: 6px;
}
